@@ -59,5 +59,8 @@ buildFlags = filter (not . null)
 #endif
 #ifdef WITH_CRYPTOHASH
 	, "CryptoHash"
+#endif
+#ifdef WITH_EKG
+	, "EKG"
 #endif
 	]
