@@ -18,7 +18,7 @@ import qualified Remote
 import qualified Backend
 
 def :: [Command]
-def = [noCommit $ noRepo showPackageVersion $ dontCheck repoExists $
+def = [noCommit $ noRepo startNoRepo $ dontCheck repoExists $
 	command "version" paramNothing seek SectionQuery "show version info"]
 
 seek :: [CommandSeek]
@@ -37,6 +37,9 @@ start = do
 			unwords upgradableVersions
 	stop
 
+startNoRepo :: CmdParams -> IO ()
+startNoRepo _ = showPackageVersion
+
 showPackageVersion :: IO ()
 showPackageVersion = do
 	info "git-annex version" SysConfig.packageversion
